@@ -8,8 +8,8 @@ pymatgen objects for analysis purposes.
 """
 
 __author__ = "Shyue Ping Ong"
-__date__ = "Mar 1 2013"
-__version__ = "0.3.4"
+__date__ = "Sep 20 2013"
+__version__ = "0.3.5"
 
 
 from query_engine import QueryEngine
