@@ -22,6 +22,7 @@ import datetime
 import string
 import json
 import socket
+import numpy as np
 from fnmatch import fnmatch
 from collections import OrderedDict
 
@@ -130,7 +131,7 @@ class VaspToDbTaskDrone(AbstractDrone):
         self.update_duplicates = update_duplicates
         self.mapi_key = mapi_key
         if not simulate_mode:
-            conn = MongoClient(self.host, self.port)
+            conn = MongoClient(self.host, self.port, j=True)
             db = conn[self.database]
             if self.user:
                 db.authenticate(self.user, self.password)
@@ -255,13 +256,10 @@ class VaspToDbTaskDrone(AbstractDrone):
                     for calc in d["calculations"]:
                         if "dos" in calc:
                             dos = json.dumps(calc["dos"])
-                            if not self.simulate:
-                                fs = gridfs.GridFS(db, "dos_fs")
-                                dosid = fs.put(dos)
-                                calc["dos_fs_id"] = dosid
-                                del calc["dos"]
-                            else:
-                                logger.info("Simulated Insert DOS into db.")
+                            fs = gridfs.GridFS(db, "dos_fs")
+                            dosid = fs.put(dos)
+                            calc["dos_fs_id"] = dosid
+                            del calc["dos"]
 
                 d["last_updated"] = datetime.datetime.today()
                 if result is None:
@@ -272,18 +270,19 @@ class VaspToDbTaskDrone(AbstractDrone):
                         )["c"]
                     logger.info("Inserting {} with taskid = {}"
                                 .format(d["dir_name"], d["task_id"]))
-                    coll.insert(d, safe=True)
                 elif self.update_duplicates:
                     d["task_id"] = result["task_id"]
                     logger.info("Updating {} with taskid = {}"
                                 .format(d["dir_name"], d["task_id"]))
-                    coll.update({"dir_name": d["dir_name"]}, {"$set": d})
+
+                coll.update({"dir_name": d["dir_name"]}, {"$set": d},
+                            upsert=True)
                 return d["task_id"]
             else:
                 logger.info("Skipping duplicate {}".format(d["dir_name"]))
         else:
             d["task_id"] = 0
-            logger.info("Simulated Insert into database for {} with task_id {}"
+            logger.info("Simulated insert into database for {} with task_id {}"
                         .format(d["dir_name"], d["task_id"]))
             return d
 
@@ -351,13 +350,17 @@ class VaspToDbTaskDrone(AbstractDrone):
 
         # Parse OUTCAR for additional information and run stats that are
         # generally not in vasprun.xml.
-        run_stats = {}
-        for filename in glob.glob(os.path.join(fullpath, "OUTCAR*")):
-            outcar = Outcar(filename)
-            i = 1 if re.search("relax2", filename) else 0
-            taskname = "relax2" if re.search("relax2", filename) else "relax1"
-            d["calculations"][i]["output"]["outcar"] = outcar.to_dict
-            run_stats[taskname] = outcar.run_stats
+        try:
+            run_stats = {}
+            for filename in glob.glob(os.path.join(fullpath, "OUTCAR*")):
+                outcar = Outcar(filename)
+                i = 1 if re.search("relax2", filename) else 0
+                taskname = "relax2" if re.search("relax2", filename) else \
+                    "relax1"
+                d["calculations"][i]["output"]["outcar"] = outcar.to_dict
+                run_stats[taskname] = outcar.run_stats
+        except:
+            logger.error("Bad OUTCAR for {}.".format(fullpath))
 
         try:
             overall_run_stats = {}
@@ -396,9 +399,9 @@ class VaspToDbTaskDrone(AbstractDrone):
                     d["incar"] = incar.to_dict
                     d["is_hubbard"] = incar.get("LDAU", False)
                     if d["is_hubbard"]:
-                        us = incar.get("LDAUU", [])
-                        js = incar.get("LDAUJ", [])
-                        if sum(us) == 0 and sum(js) == 0:
+                        us = np.array(incar.get("LDAUU", []))
+                        js = np.array(incar.get("LDAUJ", []))
+                        if sum(us - js) == 0:
                             d["is_hubbard"] = False
                             d["hubbards"] = {}
                     else:
@@ -536,11 +539,13 @@ class VaspToDbTaskDrone(AbstractDrone):
                     else "unsuccessful"
             else:
                 d["state"] = "stopped"
+
             d["analysis"] = get_basic_analysis_and_error_checks(d)
 
             sg = SymmetryFinder(Structure.from_dict(d["output"]["crystal"]),
                                 0.1)
-            d["spacegroup"] = {"symbol": sg.get_spacegroup_symbol(),
+            d["spacegroup"] = {"symbol": unicode(sg.get_spacegroup_symbol(),
+                                                 errors="ignore"),
                                "number": sg.get_spacegroup_number(),
                                "point_group": unicode(sg.get_point_group(),
                                                       errors="ignore"),
@@ -580,9 +585,9 @@ class VaspToDbTaskDrone(AbstractDrone):
     def __str__(self):
         return "VaspToDbDictDrone"
 
-    @staticmethod
-    def from_dict(d):
-        return VaspToDbTaskDrone(**d["init_args"])
+    @classmethod
+    def from_dict(cls, d):
+        return cls(**d["init_args"])
 
     @property
     def to_dict(self):
@@ -599,7 +604,9 @@ class VaspToDbTaskDrone(AbstractDrone):
         return output
 
 
-def get_basic_analysis_and_error_checks(d):
+def get_basic_analysis_and_error_checks(d, max_force_threshold=0.5,
+                                        volume_change_threshold=0.2):
+
     initial_vol = d["input"]["crystal"]["lattice"]["volume"]
     final_vol = d["output"]["crystal"]["lattice"]["volume"]
     delta_vol = final_vol - initial_vol
@@ -611,10 +618,12 @@ def get_basic_analysis_and_error_checks(d):
     vbm = calc["output"]["vbm"]
     is_direct = calc["output"]["is_gap_direct"]
 
-    if abs(percent_delta_vol) > 0.20:
-        warning_msgs = ["Volume change > 20%"]
-    else:
-        warning_msgs = []
+    warning_msgs = []
+    error_msgs = []
+
+    if abs(percent_delta_vol) > volume_change_threshold:
+        warning_msgs.append("Volume change > {}%"
+                            .format(volume_change_threshold * 100))
 
     bv_struct = Structure.from_dict(d["output"]["crystal"])
     try:
@@ -626,9 +635,30 @@ def get_basic_analysis_and_error_checks(d):
     except Exception as ex:
         logger.error("BVAnalyzer error {e}.".format(e=str(ex)))
 
+    max_force = None
+    if d["state"] == "successful" and \
+            d["calculations"][0]["input"]["parameters"].get("NSW", 0) > 0:
+        # handle the max force and max force error
+        max_force = max([np.linalg.norm(a)
+                        for a in d["calculations"][-1]["output"]
+                        ["ionic_steps"][-1]["forces"]])
+
+        if max_force > max_force_threshold:
+            error_msgs.append("Final max force exceeds {} eV"
+                              .format(max_force_threshold))
+            d["state"] = "error"
+
+        s = Structure.from_dict(d["output"]["crystal"])
+        if not s.is_valid():
+            error_msgs.append("Bad structure (atoms are too close!)")
+            d["state"] = "error"
+
     return {"delta_volume": delta_vol,
+            "max_force": max_force,
             "percent_delta_volume": percent_delta_vol,
-            "warnings": warning_msgs, "coordination_numbers": coord_num,
+            "warnings": warning_msgs,
+            "errors": error_msgs,
+            "coordination_numbers": coord_num,
             "bandgap": gap, "cbm": cbm, "vbm": vbm,
             "is_gap_direct": is_direct,
             "bv_structure": bv_struct.to_dict}
