@@ -108,7 +108,38 @@ class QueryEngine(object):
         self.db = self.connection[database]
         if user:
             self.db.authenticate(user, password)
+        self.set_collection(collection=collection)
+        self.set_aliases_and_defaults(aliases_config=aliases_config,
+                                      default_properties=default_properties)
+
+    def set_collection(self, collection):
+        """
+        Switch to another collection. Note that you may have to set the
+        aliases and default properties via set_aliases_and_defaults if the
+        schema of the new collection differs from the current collection.
+
+        Args:
+            collection:
+                Name of collection.
+        """
         self.collection = self.db[collection]
+
+    def set_aliases_and_defaults(self, aliases_config=None,
+                                 default_properties=None):
+        """
+        Set the alias config and defaults to use. Typically used when
+        switching to a collection with a different schema.
+
+        Args:
+            aliases_config:
+                An alias dict to use. Defaults to None, which means the default
+                aliases defined in "aliases.json" is used. See constructor
+                for format.
+            default_properties:
+                List of property names (strings) to use by default, if no
+                properties are given to the 'properties' argument of
+                query().
+        """
         if aliases_config is None:
             with open(os.path.join(os.path.dirname(__file__),
                                    "aliases.json")) as f:
@@ -406,6 +437,12 @@ class QueryEngine(object):
                 user=user, password=password, collection=d["collection"],
                 aliases_config=d.get("aliases_config", None))
 
+    def __getitem__(self, item):
+        """Support pymongo.Database syntax db['collection'] to access collections.
+        Simply delegate this to the pymongo.Database instance, so behavior is the same.
+        """
+        return self.db[item]
+
 
 class QueryResults(Iterable):
     """
@@ -465,17 +502,6 @@ class QueryResults(Iterable):
 
 class QueryError(Exception):
     """
-    Exception class for QueryEngine. Allows more information exception messages
-    to cover situations not covered by standard exception classes.
+    Exception class for errors occuring during queries.
     """
-
-    def __init__(self, msg):
-        """
-        Args:
-            msg:
-                Message for the error.
-        """
-        self.msg = msg
-
-    def __str__(self):
-        return "Query Error : " + self.msg
+    pass
