@@ -19,7 +19,7 @@ class Report:
         """Create blank report, with a header.
 
         :param header: Report header
-        :type header: ReportHeader
+        :type header: Header
         """
         self._hdr = header
         self._sections = []
@@ -34,11 +34,16 @@ class Report:
     def is_empty(self):
         if len(self._sections) == 0:
             return True
-        total_rows = 0
+        self._total_rows = 0
         for sect in self._sections:
-            if sect.body is not None:
-                total_rows += sect.body.nrow
-        return total_rows == 0
+            self._count_rows(sect)
+        return self._total_rows == 0
+
+    def _count_rows(self, sect):
+        if sect.body is not None:
+            self._total_rows += sect.body.nrow
+        for subsect in sect._sections:
+            self._count_rows(subsect)
 
     def __iter__(self):
         return iter(self._sections)
@@ -62,6 +67,7 @@ class ReportSection(Report):
     def body(self):
         return self._body
 
+
 class Header:
     """Base header class.
     """
@@ -79,7 +85,8 @@ class Header:
         return iter(self._kv)
 
     def to_dict(self):
-        return {k:v for k, v in self._kv}
+        return {k: v for k, v in self._kv}
+
 
 class ReportHeader(Header):
     """Header for entire report.
@@ -269,11 +276,11 @@ class JSONFormatter:
             sections=[
                 dict(title=s.header.title,
                      info=s.header,
-                     conditions=[
-                        dict(title=cs.header.title,
-                             info=cs.header,
-                             violations=cs.body)
-                        for cs in s
+                     conditions=[dict(
+                         title=cs.header.title,
+                         info=cs.header,
+                         violations=cs.body)
+                         for cs in s
                      ]
                 )
                 for s in report
@@ -281,6 +288,7 @@ class JSONFormatter:
         )
         return json.dumps(obj, indent=self._indent, cls=MongoJSONEncoder)
 
+
 class ReportJSONEncoder(MongoJSONEncoder):
     def default(self, o):
         if isinstance(o, Header):
@@ -300,30 +308,43 @@ class MarkdownFormatter:
         return ', '.join((('{}={}'.format(k, v) for k, v in d.iteritems())))
 
     def _fixed_width(self, values, widths):
-        s = ''.join(["{{:{:d}s}}".format(w+1).format(str(v))
-                       for w, v in zip(widths, values)])
+        s = ''.join(["{{:{:d}s}}".format(w + 1).format(str(v))
+                    for w, v in zip(widths, values)])
         return s
 
     def format(self, report):
         lines = []
-        lines.append('# {} #\n'.format(report.header.title))
-        lines.append('Info: {}'.format(self._mapdump(report.header.to_dict())))
+        self._append_heading(lines, 1, report.header.title)
+        self._append_info_section(lines, report.header)
         for section in report:
-            lines.append('\n## {} ##\n'.format(section.header.title))
-            if section.header:
-                lines.append('Info: {}'.format(self._mapdump(section.header.to_dict())))
+            self._append_heading(lines, 2, section.header.title)
+            self._append_info_section(lines, section.header)
             for cond in section:
-                lines.append('\n### {} ###\n'.format(cond.header.title))
-                if cond.header:
-                    lines.append('Info: {}'.format(self._mapdump(cond.header.to_dict())))
-                lines.append('\nViolations:\n')
-                indent = '    '
-                tbl = cond.body
-                lines.append(indent + self._fixed_width(tbl.column_names, tbl.column_widths))
-                for row in tbl:
-                    lines.append(indent + self._fixed_width(row, tbl.column_widths))
+                self._append_heading(lines, 3, cond.header.title)
+                self._append_info_section(lines, cond.header)
+                self._append_violations(lines, cond.body)
         return '\n'.join(lines)
 
+    def _append_info_section(self, lines, info):
+        if not info:
+            return
+        infodict = info.to_dict()
+        if infodict:
+            text = 'Info: {}'.format(self._mapdump(infodict))
+            lines.append(text)
+
+    def _append_heading(self, lines, level, title):
+        hashes = '#' * level
+        text = '\n{} {} {}\n'.format(hashes, title, hashes)
+        lines.append(text)
+
+    def _append_violations(self, lines, data):
+        lines.append('\nViolations:\n')
+        indent = '    '
+        lines.append(indent + self._fixed_width(data.column_names, data.column_widths))
+        for row in data:
+            lines.append(indent + self._fixed_width(row, data.column_widths))
+
 
 class Emailer(DoesLogging):
     """Send a report to an email recipient.
@@ -373,4 +394,4 @@ class Emailer(DoesLogging):
         except Exception, err:
             self._log.error("connection to SMTP server failed: {}".format(err))
             n_recip = 0
-        return n_recip
\ No newline at end of file
+        return n_recip
